@@ -1,10 +1,13 @@
 import sys
+import warnings
 from dataclasses import dataclass, field
 from difflib import SequenceMatcher
 from typing import Dict, List, NamedTuple, Optional
 
 import numpy as np
 
+from .thermodynamic_restrictions import ComplexFormation, DuplicateError, ThermodynamicRestrictions
+
 
 class UnregisteredRule(NamedTuple):
     expected: Optional[str]
@@ -40,24 +43,72 @@ PREPOSITIONS: List[str] = [
 
 
 @dataclass
-class ReactionRules(object):
+class ReactionRules(ThermodynamicRestrictions):
     """Create an executable biochemical model from text.
 
-    **reaction** | **parameters** | **initial conditions**
+    .. list-table:: Available reaction rules
+        :widths: 25 50 25
+        :header-rows: 1
+
+        * - Rule
+          - Example sentence
+          - Parameters (optional)
+        * - :func:`~pasmopy.construction.reaction_rules.dimerize`
+          - *A* dimerizes <--> *AA*
+          - .. math:: kf, kr
+        * - :func:`~pasmopy.construction.reaction_rules.bind`
+          - *A* binds *B* <--> *AB*
+          - .. math:: kf, kr
+        * - :func:`~pasmopy.construction.reaction_rules.dissociate`
+          - *AB* dissociates to *A* and *B*
+          - .. math:: kf, kr
+        * - :func:`~pasmopy.construction.reaction_rules.is_phosphorylated`
+          - *uA* is phosphorylated <--> *pA*
+          - .. math:: kf, kr
+        * - :func:`~pasmopy.construction.reaction_rules.is_dephosphorylated`
+          - *pA* is dephosphorylated --> *uA*
+          - .. math:: V, K
+        * - :func:`~pasmopy.construction.reaction_rules.phosphorylate`
+          - *B* phosphorylates *uA* --> *pA*
+          - .. math:: V, K
+        * - :func:`~pasmopy.construction.reaction_rules.dephosphorylate`
+          - *B* dephosphorylates *pA* --> *uA*
+          - .. math:: V, K
+        * - :func:`~pasmopy.construction.reaction_rules.transcribe`
+          - *B* transcribes *a*
+          - .. math:: V, K, n, (KF, nF)
+        * - :func:`~pasmopy.construction.reaction_rules.is_translated`
+          - *a* is translated into *A*
+          - .. math:: kf
+        * - :func:`~pasmopy.construction.reaction_rules.synthesize`
+          - *B* synthesizes *A*
+          - .. math:: kf
+        * - :func:`~pasmopy.construction.reaction_rules.is_synthesized`
+          - *A* is synthesized
+          - .. math:: kf
+        * - :func:`~pasmopy.construction.reaction_rules.degrade`
+          - *B* degrades *A*
+          - .. math:: kf
+        * - :func:`~pasmopy.construction.reaction_rules.is_degraded`
+          - *A* is degraded
+          - .. math:: kf
+        * - :func:`~pasmopy.construction.reaction_rules.translocate`
+          - *Acyt* translocates from cytoplasm to nucleus (Vcyt, Vnuc) <--> *Anuc*
+          - .. math:: kf, kr, (V_{pre}, V_{post})
 
     Attributes
     ----------
     input_txt : str
         Model description file (*.txt), e.g.,
-        `Kholodenko_JBC_1999.txt <https://github.com/pasmopy/pasmopy/blob/master/tests/text_files/Kholodenko_JBC_1999.txt>`_.
+        `Kholodenko1999.txt <https://github.com/pasmopy/pasmopy/blob/master/tests/text_files/Kholodenko1999.txt>`_.
     parameters : list of strings
-        x : model parameters.
+        ``x`` : model parameters.
     species : list of strings
-        y : model species.
+        ``y`` : model species.
     reactions : list of strings
-        v : flux vector.
+        ``v`` : flux vector.
     differential_equations : list of strings
-        dydt : right-hand side of the differential equation.
+        ``dydt`` : right-hand side of the differential equation.
     obs_desc : list of List[str]
         Description of observables.
     param_info : list of strings
@@ -78,6 +129,8 @@ class ReactionRules(object):
         Untreated conditions to get steady state.
     rule_words : dict
         Words to identify reaction rules.
+    complex_formations : list
+        List of ComplexFormation to detect duplicate binding-dissociation.
 
     """
 
@@ -149,7 +202,7 @@ class ReactionRules(object):
                 " binds",
                 " forms complexes with",
             ],
-            is_dissociated=[
+            dissociate=[
                 " is dissociated into",
                 " dissociates to",
             ],
@@ -340,7 +393,7 @@ class ReactionRules(object):
                     else:
                         raise NameError(
                             f"line{line_num:d}: "
-                            "Name'{ival.split('=')[0].strip(' ')}' is not defined."
+                            f"Name'{ival.split('=')[0].strip(' ')}' is not defined."
                         )
             line = line.split("|")[0]
         hit_words: List[str] = []
@@ -435,35 +488,45 @@ class ReactionRules(object):
         """
         Examples
         --------
-        >>> 'Monomer dimerizes --> Dimer'
-        >>> 'Monomer homodimerizes --> Dimer'
-        >>> 'Monomer forms a dimer --> Dimer'
-        >>> 'Monomer forms dimers --> Dimer'
+        >>> 'A dimerizes <--> AA'
+        >>> 'A homodimerizes <--> AA'
+        >>> 'A forms a dimer <--> AA'
+        >>> 'A forms dimers <--> AA'
 
         Notes
         -----
+        * Parameters
+            .. math:: kf, kr
+
         * Rate equation
-            .. math:: v = kf * [Monomer] * [Monomer] - kr * [Dimer]
+            .. math:: v = kf * [A] * [A] - kr * [AA]
 
         * Differential equation
             .. math::
 
-                d[Monomer]/dt = - 2 * v
+                d[A]]/dt = - 2 * v
 
-                d[Dimer]/dt = + v
+                d[AA]/dt = + v
 
         """
         description = self._preprocessing(
             sys._getframe().f_code.co_name, line_num, line, "kf", "kr"
         )
         monomer = description[0].strip(" ")
-        if " --> " in description[1]:
+        if " <--> " in description[1]:
+            dimer = description[1].split(" <--> ")[1].strip(" ")
+        elif " --> " in description[1]:
+            warnings.warn(
+                f"line{line_num:d}: Use '<-->' instead of '-->' for reversible reaction rules.",
+                FutureWarning,
+            )
             dimer = description[1].split(" --> ")[1].strip(" ")
         else:
-            raise ValueError(f"line{line_num:d}: Use '-->' to specify the name of the dimer.")
+            raise ValueError(f"line{line_num:d}: Use '<-->' to specify the name of the dimer.")
         if monomer == dimer:
             raise ValueError(f"{dimer} <- Use a different name.")
         self._set_species(monomer, dimer)
+        self.complex_formations.append(ComplexFormation(line_num, set(monomer), dimer, True))
         self.reactions.append(
             f"v[{line_num:d}] = "
             f"x[C.kf{line_num:d}] * y[V.{monomer}] * y[V.{monomer}] - "
@@ -486,35 +549,45 @@ class ReactionRules(object):
         """
         Examples
         --------
-        >>> 'Component1 binds Component2 --> Complex'
-        >>> 'Component1 forms complexes with Component2 --> Complex'
+        >>> 'A binds B <--> AB'
+        >>> 'A forms complexes with B <--> AB'
 
         Notes
         -----
+        * Parameters
+            .. math:: kf, kr
+
         * Rate equation
-            .. math:: v = kf * [Component1] * [Component2] - kr * [Complex]
+            .. math:: v = kf * [A] * [B] - kr * [AB]
 
         * Differential equation
             .. math::
 
-                d[Component1]/dt = - v
+                d[A]/dt = - v
 
-                d[Component2]/dt = - v
+                d[B]/dt = - v
 
-                d[Complex]/dt = + v
+                d[AB]/dt = + v
 
         """
         description = self._preprocessing(
             sys._getframe().f_code.co_name, line_num, line, "kf", "kr"
         )
         component1 = description[0].strip(" ")
-        if " --> " in description[1]:
+        if " <--> " in description[1]:
             # Specify name of the complex
+            component2 = description[1].split(" <--> ")[0].strip(" ")
+            complex = description[1].split(" <--> ")[1].strip(" ")
+        elif " --> " in description[1]:
+            warnings.warn(
+                f"line{line_num:d}: Use '<-->' instead of '-->' for reversible reaction rules.",
+                FutureWarning,
+            )
             component2 = description[1].split(" --> ")[0].strip(" ")
             complex = description[1].split(" --> ")[1].strip(" ")
         else:
             raise ValueError(
-                f"line{line_num:d}: Use '-->' to specify the name of the protein complex."
+                f"line{line_num:d}: Use '<-->' to specify the name of the protein complex."
             )
         if component1 == complex or component2 == complex:
             raise ValueError(f"line{line_num:d}: {complex} <- Use a different name.")
@@ -522,6 +595,9 @@ class ReactionRules(object):
             self.dimerize(line_num, line)
         else:
             self._set_species(component1, component2, complex)
+            self.complex_formations.append(
+                ComplexFormation(line_num, set([component1, component2]), complex, True)
+            )
             self.reactions.append(
                 f"v[{line_num:d}] = "
                 f"x[C.kf{line_num:d}] * y[V.{component1}] * y[V.{component2}] - "
@@ -545,26 +621,29 @@ class ReactionRules(object):
             if counter_complex == 0:
                 self.differential_equations.append(f"dydt[V.{complex}] = + v[{line_num:d}]")
 
-    def is_dissociated(self, line_num: int, line: str) -> None:
+    def dissociate(self, line_num: int, line: str) -> None:
         """
         Examples
         --------
-        >>> 'Complex is dissociated into Component1 and Component2'
-        >>> 'Complex dissociates to Component1 and Component2'
+        >>> 'AB dissociates to A and B'
+        >>> 'AB is dissociated into A and B'
 
         Notes
         -----
+        * Parameters
+            .. math:: kf, kr
+
         * Rate equation
-            .. math:: v = kf * [Complex] - kr * [Component1] * [Component2]
+            .. math:: v = kf * [AB] - kr * [A] * [B]
 
         * Differential equation
             .. math::
 
-                d[Component1]/dt = + v
+                d[A]/dt = + v
 
-                d[Component2]/dt = + v
+                d[B]/dt = + v
 
-                d[Complex]/dt = - v
+                d[AB]/dt = - v
 
         """
         description = self._preprocessing(
@@ -579,6 +658,9 @@ class ReactionRules(object):
             component1 = description[1].split(" and ")[0].strip(" ")
             component2 = description[1].split(" and ")[1].strip(" ")
         self._set_species(complex, component1, component2)
+        self.complex_formations.append(
+            ComplexFormation(line_num, set([component1, component2]), complex, False)
+        )
         self.reactions.append(
             f"v[{line_num:d}] = "
             f"x[C.kf{line_num:d}] * y[V.{complex}] - "
@@ -610,31 +692,40 @@ class ReactionRules(object):
         """
         Examples
         --------
-        >>> 'uProtein is phosphorylated --> pProtein'
+        >>> 'uA is phosphorylated <--> pA'
 
         Notes
         -----
+        * Parameters
+            .. math:: kf, kr
+
         * Rate equation
-            .. math:: v = kf * [uProtein] - kr * [pProtein]
+            .. math:: v = kf * [uA] - kr * [pA]
 
         * Differential equation
             .. math::
 
-                d[uProtein]/dt = - v
+                d[uA]/dt = - v
 
-                d[pProtein]/dt = + v
+                d[pA]/dt = + v
 
         """
         description = self._preprocessing(
             sys._getframe().f_code.co_name, line_num, line, "kf", "kr"
         )
         unphosphorylated_form = description[0].strip(" ")
-        if " --> " in description[1]:
+        if " <--> " in description[1]:
+            phosphorylated_form = description[1].split(" <--> ")[1].strip(" ")
+        elif " --> " in description[1]:
+            warnings.warn(
+                f"line{line_num:d}: Use '<-->' instead of '-->' for reversible reaction rules.",
+                FutureWarning,
+            )
             phosphorylated_form = description[1].split(" --> ")[1].strip(" ")
         else:
             raise ValueError(
                 f"line{line_num:d}: "
-                "Use '-->' to specify the name of the phosphorylated protein."
+                "Use '<-->' to specify the name of the phosphorylated protein."
             )
         self._set_species(unphosphorylated_form, phosphorylated_form)
 
@@ -664,19 +755,22 @@ class ReactionRules(object):
         """
         Examples
         --------
-        >>> 'pProtein is dephosphorylated --> uProtein'
+        >>> 'pA is dephosphorylated --> uA'
 
         Notes
         -----
+        * Parameters
+            .. math:: V, K
+
         * Rate equation
-            .. math:: v = V * [pProtein] / (K + [pProtein])
+            .. math:: v = V * [pA] / (K + [pA])
 
         * Differential equation
             .. math::
 
-                d[uProtein]/dt = + v
+                d[uA]/dt = + v
 
-                d[pProtein]/dt = - v
+                d[pA]/dt = - v
 
         """
         description = self._preprocessing(sys._getframe().f_code.co_name, line_num, line, "V", "K")
@@ -716,19 +810,22 @@ class ReactionRules(object):
         """
         Examples
         --------
-        >>> 'Kinase phosphorylates uProtein --> pProtein'
+        >>> 'B phosphorylates uA --> pA'
 
         Notes
         -----
+        * Parameters
+            .. math:: V, K
+
         * Rate equation
-            .. math:: v = V * [Kinase] * [uProtein] / (K + [uProtein])
+            .. math:: v = V * [B] * [uA] / (K + [uA])
 
         * Differential equation
             .. math::
 
-                d[uProtein]/dt = - v
+                d[uA]/dt = - v
 
-                d[pProtein]/dt = + v
+                d[pA]/dt = + v
 
         """
         description = self._preprocessing(sys._getframe().f_code.co_name, line_num, line, "V", "K")
@@ -772,19 +869,22 @@ class ReactionRules(object):
         """
         Examples
         --------
-        >>> 'Phosphatase dephosphorylates pProtein --> uProtein'
+        >>> 'B dephosphorylates pA --> uA'
 
         Notes
         -----
+        * Parameters
+            .. math:: V, K
+
         * Rate equation
-            .. math:: v = V * [phosphatase] * [pProtein] / (K + [pProtein])
+            .. math:: v = V * [B] * [pA] / (K + [pA])
 
         * Differential equation
             .. math::
 
-                d[uProtein]/dt = + v
+                d[uA]/dt = + v
 
-                d[pProtein]/dt = - v
+                d[pA]/dt = - v
 
         """
         description = self._preprocessing(sys._getframe().f_code.co_name, line_num, line, "V", "K")
@@ -828,23 +928,26 @@ class ReactionRules(object):
         """
         Examples
         --------
-        >>> 'TF transcribes mRNA'
-        >>> 'TF1 & TF2 transcribe mRNA'  # (AND-gate)
-        >>> 'TF transcribes mRNA, repressed by Repressor'  # (Negative regulation)
+        >>> 'B transcribes a'
+        >>> 'B1 & B2 transcribe a'  # (AND-gate)
+        >>> 'B transcribes a, repressed by C'  # (Negative regulation)
 
         Notes
         -----
+        * Parameters
+            .. math:: V, K, n, (KF, nF)
+
         * Rate equation
             .. math::
 
-                v = V * [TF] ^ {n} / (K ^ {n} + [TF] ^ {n})
+                v = V * [B] ^ {n} / (K ^ {n} + [B] ^ {n})
 
-                v = V * ([TF1] * [TF2]) ^ {n} / (K ^ {n} + ([TF1] * [TF2]) ^ {n})
+                v = V * ([B1] * [B2]) ^ {n} / (K ^ {n} + ([B1] * [B2]) ^ {n})
 
-                v = V * [TF] ^ {n} / (K ^ {n} + [TF] ^ {n} + ([Repressor] / KF) ^ {nF})
+                v = V * [B] ^ {n} / (K ^ {n} + [B] ^ {n} + ([C] / KF) ^ {nF})
 
         * Differential equation
-            .. math:: d[mRNA]/dt = + v
+            .. math:: d[a]/dt = + v
 
         """
         description = self._preprocessing(
@@ -911,15 +1014,18 @@ class ReactionRules(object):
         """
         Examples
         --------
-        >>> 'mRNA is translated into Protein'
+        >>> 'a is translated into A'
 
         Notes
         -----
+        * Parameters
+            .. math:: kf
+
         * Rate equation
-            .. math:: v = kf * [mRNA]
+            .. math:: v = kf * [a]
 
         * Differential equation
-            .. math:: d[Protein]/dt = + v
+            .. math:: d[A]/dt = + v
 
         """
         description = self._preprocessing(sys._getframe().f_code.co_name, line_num, line, "kf")
@@ -939,15 +1045,18 @@ class ReactionRules(object):
         """
         Examples
         --------
-        >>> 'Catalyst synthesizes Product'
+        >>> 'B synthesizes A'
 
         Notes
         -----
+        * Parameters
+            .. math:: kf
+
         * Rate equation
-            .. math:: v = kf * [Catalyst]
+            .. math:: v = kf * [B]
 
         * Differential equation
-            .. math:: d[Product]/dt = + v
+            .. math:: d[A]/dt = + v
 
         """
         description = self._preprocessing(sys._getframe().f_code.co_name, line_num, line, "kf")
@@ -967,15 +1076,18 @@ class ReactionRules(object):
         """
         Examples
         --------
-        >>> 'ChemicalSpecies is synthesized'
+        >>> 'A is synthesized'
 
         Notes
         -----
+        * Parameters
+            .. math:: kf
+
         * Rate equation
             .. math:: v = kf
 
         * Differential equation
-            .. math:: d[ChemicalSpecies]/dt = + v
+            .. math:: d[A]/dt = + v
 
         """
         description = self._preprocessing(sys._getframe().f_code.co_name, line_num, line, "kf")
@@ -994,15 +1106,18 @@ class ReactionRules(object):
         """
         Examples
         --------
-        >>> 'Protease degrades Protein'
+        >>> 'B degrades A'
 
         Notes
         -----
+        * Parameters
+            .. math:: kf
+
         * Rate equation
-            .. math:: v = kf * [Protease]
+            .. math:: v = kf * [B]
 
         * Differential equation
-            .. math:: d[Protein]/dt = - v
+            .. math:: d[A]/dt = - v
 
         """
         description = self._preprocessing(sys._getframe().f_code.co_name, line_num, line, "kf")
@@ -1022,15 +1137,18 @@ class ReactionRules(object):
         """
         Examples
         --------
-        >>> 'ChemicalSpecies is degraded'
+        >>> 'A is degraded'
 
         Notes
         -----
+        * Parameters
+            .. math:: kf
+
         * Rate equation
-            .. math:: v = kf * [ChemicalSpecies]
+            .. math:: v = kf * [A]
 
         * Differential equation
-            .. math:: d[ChemicalSpecies]/dt = - v
+            .. math:: d[A]/dt = - v
 
         """
         description = self._preprocessing(sys._getframe().f_code.co_name, line_num, line, "kf")
@@ -1049,32 +1167,41 @@ class ReactionRules(object):
         r"""
         Examples
         --------
-        >>> 'pre_translocation translocates from one location to another (pre_volume, post_volume) --> post_translocation'
-        >>> 'pre_translocation is translocated from one location to another (pre_volume, post_volume) --> post_translocation'
+        >>> 'A_at_cyt translocates from cytoplasm to nucleus (V_cyt, V_nuc) <--> A_at_nuc'
+        >>> 'A_at_cyt is translocated from cytoplasm to nucleus (V_cyt, V_nuc) <--> A_at_nuc'
 
         Notes
         -----
+        * Parameters
+            .. math:: kf, kr, (V_{pre}, V_{post})
+
         * Rate equation
-            .. math:: v = kf * [pre\_translocation] - kr * (V_{post} / V_{pre}) * [post\_translocation]
+            .. math:: v = kf * [A\_at\_pre] - kr * (V_{post} / V_{pre}) * [A\_at\_post]
 
         * Differential equation
             .. math::
 
-                d[pre\_translocation]/dt = - v
+                d[A\_at\_pre]/dt = - v
 
-                d[post\_translocation]/dt = + v * (V_{pre} / V_{post})
+                d[A\_at\_post]/dt = + v * (V_{pre} / V_{post})
 
         """
         description = self._preprocessing(
             sys._getframe().f_code.co_name, line_num, line, "kf", "kr"
         )
         pre_translocation = description[0].strip(" ")
-        if " --> " in description[1]:
+        if " <--> " in description[1]:
+            post_translocation = description[1].split(" <--> ")[1].strip(" ")
+        elif " --> " in description[1]:
+            warnings.warn(
+                f"line{line_num:d}: Use '<-->' instead of '-->' for reversible reaction rules.",
+                FutureWarning,
+            )
             post_translocation = description[1].split(" --> ")[1].strip(" ")
         else:
             raise ValueError(
                 f"line{line_num:d}: "
-                "Use '-->' to specify the name of the species after translocation."
+                "Use '<-->' to specify the name of the species after translocation."
             )
         if pre_translocation == post_translocation:
             raise ValueError(f"line{line_num:d}: {post_translocation} <- Use a different name.")
@@ -1143,7 +1270,7 @@ class ReactionRules(object):
                 continue
             elif lines.count(line) > 1:
                 # Find duplicate lines
-                raise ValueError(
+                raise DuplicateError(
                     f"Reaction '{line}' is duplicated in lines "
                     + ", ".join([str(i + 1) for i, rxn in enumerate(lines) if rxn == line])
                 )
@@ -1151,7 +1278,10 @@ class ReactionRules(object):
             elif line.startswith("@obs "):
                 line = self._remove_prefix(line, "@obs ")
                 if line.count(":") != 1:
-                    raise SyntaxError(f"line{line_num:d}: Missing colon")
+                    raise SyntaxError(
+                        f"line{line_num:d}: Missing colon\n"
+                        "Should be `@obs <observable name>: <expression>`."
+                    )
                 else:
                     self.obs_desc.append(line.split(":"))
             # About simulation info.
@@ -1171,7 +1301,7 @@ class ReactionRules(object):
                                 raise TypeError("@sim tspan: [t0, tf] must be a list of integers.")
                         else:
                             raise ValueError(
-                                "tspan must be a two element vector [t0, tf] "
+                                "`tspan` must be a two element vector [t0, tf] "
                                 "specifying the initial and final times."
                             )
                     elif line.startswith("unperturbed"):
@@ -1183,7 +1313,7 @@ class ReactionRules(object):
                     else:
                         raise ValueError(
                             f"(line{line_num:d}) Available options are: "
-                            "'@sim tspan:', '@sim unperturbed:' or '@sim condition XXX:'."
+                            "'@sim tspan:', '@sim unperturbed:', or '@sim condition XXX:'."
                         )
             # Detect reaction rule
             else:
