@@ -2,7 +2,7 @@ import os
 import re
 import shutil
 from dataclasses import dataclass, field
-from typing import List
+from typing import Dict, List, Optional
 
 try:  # python 3.8+
     from typing import Literal
@@ -15,12 +15,15 @@ from .reaction_rules import ReactionRules
 
 @dataclass
 class Text2Model(ReactionRules):
-    """Build a BioMASS-formatted model based on template.
+    """
+    Build a BioMASS-formatted model based on template.
+
+    **reaction** | **parameters** | **initial conditions**
 
     Attributes
     ----------
     input_txt : str
-        Model description file (*.txt), e.g., 'Kholodenko_JBC_1999.txt'
+        Model description file (*.txt), e.g., 'Kholodenko1999.txt'
 
     lang : Literal["python", "julia"] (default: 'python')
         Either 'python' or 'julia'.
@@ -357,7 +360,7 @@ class Text2Model(ReactionRules):
         """
         Replace
         - p[xxx] with x[C.xxx]
-        - u[xxx] with Y[:, V.xxx]
+        - u[xxx] with sol.y[V.xxx]
         - init[xxx] with y0[V.xxx]
 
         Parameters
@@ -391,7 +394,7 @@ class Text2Model(ReactionRules):
                 raise NameError(f"{s_name.strip()} is not defined in model species.")
             else:
                 line = (
-                    line.replace(f"u[{s_name.strip()}]", f"sol.y[V.{s_name.strip()}, :]")
+                    line.replace(f"u[{s_name.strip()}]", f"sol.y[V.{s_name.strip()}]")
                     if self.lang == "python"
                     else line.replace(f"u[{s_name.strip()}]", f"sol.u[j][V.{s_name.strip()}]")
                 )
@@ -494,7 +497,7 @@ class Text2Model(ReactionRules):
                                 + f"\n{4 * self.indentation}".join(
                                     c.strip(" ") for c in condition[1].split(sep=";")
                                 )
-                                + "\n\n"
+                                + ("\n\n" if i == len(self.sim_conditions) - 1 else "")
                             )
                         # pa: parameters
                         # init: initial conditions
@@ -657,19 +660,28 @@ class Text2Model(ReactionRules):
         ) as f:
             f.write("\n".join(lines))
 
-    def convert(self, overwrite: bool = False) -> None:
+    def convert(
+        self,
+        *,
+        show_restrictions: bool = False,
+        overwrite: bool = False,
+    ) -> None:
         """
         Convert text to a biomass-formatted model.
 
         Parameters
         ----------
-        overwrite : bool (defauld: False)
-            If True, the model folder will be overwritten.
+        show_restrictions : bool (defauld: :obj:`False`)
+            Whether to display reaction indices in which thermodynamic restrictions should be
+            imposed. These detailed balance constraints require the product of the equilibrium
+            constants along a cycle to be equal to 1.
+        overwrite : bool (defauld: :obj:`False`)
+            If :obj:`True`, the model folder will be overwritten.
 
         Examples
         --------
         >>> from pasmopy import Text2Model
-        >>> Text2Model("Kholodenko_JBC_1999.txt").convert()
+        >>> Text2Model("Kholodenko1999.txt").convert()
 
         """
         if overwrite and os.path.isdir(
@@ -688,6 +700,7 @@ class Text2Model(ReactionRules):
             os.makedirs(os.path.join(f"{self.name}_jl", "name2idx"))
 
         self.create_ode()
+        self.find_cyclic_reaction_routes()
         self._update_parameters()
         self._update_species()
         self._update_set_model()
@@ -702,13 +715,18 @@ class Text2Model(ReactionRules):
                 mode="w",
             ) as f:
                 f.write("\n".join(lines))
-        print(
-            "Model Information\n"
-            "-----------------\n"
-            f"{len(self.reactions):d} reactions\n"
-            f"{len(self.species):d} species\n"
-            f"{len(self.parameters):d} parameters"
-        )
+        print("Model information\n-----------------")
+        print(f"{len(self.reactions):d} reactions")
+        print(f"{len(self.species):d} species")
+        print(f"{len(self.parameters):d} parameters")
+        if show_restrictions:
+            if len(self.restrictions) == 0:
+                print("No cyclic reaction routes.")
+            else:
+                print("\nThermodynamic restrictions")
+                print("--------------------------")
+                for restriction in self.restrictions:
+                    print("{" + ", ".join(restriction) + "}")
 
     def to_markdown(self, n_reaction: int) -> None:
         """
@@ -722,7 +740,7 @@ class Text2Model(ReactionRules):
         Examples
         --------
         >>> from pasmopy import Text2Model
-        >>> Text2Model("Kholodenko_JBC_1999.txt").to_markdown(25)
+        >>> Text2Model("Kholodenko1999.txt").to_markdown(25)
 
         """
         os.makedirs(os.path.join("markdown", f"{self.name.split(os.sep)[-1]}"), exist_ok=True)
@@ -799,36 +817,62 @@ class Text2Model(ReactionRules):
                 ]
             )
 
-    def register_word(self, rxn_rule: str, my_word: str) -> None:
+    def register_word(
+        self,
+        terminology: Optional[
+            Dict[
+                Literal[
+                    "dimerize",
+                    "bind",
+                    "dissociate",
+                    "is_phosphorylated",
+                    "is_dephosphorylated",
+                    "phosphorylate",
+                    "dephosphorylate",
+                    "transcribe",
+                    "is_translated",
+                    "synthesize",
+                    "is_synthesized",
+                    "degrade",
+                    "is_degraded",
+                    "translocate",
+                ],
+                List[str],
+            ]
+        ] = None,
+    ) -> None:
         """
         Register user-defined rule word.
 
         Parameters
         ----------
-        rxn_rule : str
-            Define a rule to which users register a new rule word.
-
-        my_word : str
-            User-defined rule word.
+        terminology : Dict[str, List[str]], optional
+            Rule to which users register a new rule word and list of user-defined rule words.
 
         Examples
         --------
         >>> from pasmopy import Text2Model
         >>> mm_kinetics = Text2Model("michaelis_menten.txt")
-        >>> mm_kinetics.register_word("is_dissociated", "releases")
+        >>> mm_kinetics.register_word({"dissociate": ["releases"]})
         >>> mm_kinetics.convert()
 
         """
-        if rxn_rule not in self.rule_words.keys():
-            raise ValueError(
-                f"{rxn_rule} is not defined in reaction_rules.\n"
-                f"Choose a reaction rule from {', '.join(map(str, self.rule_words.keys()))}"
-            )
-        for rule, words in self.rule_words.items():
-            for registered_word in words:
-                if " " + my_word in registered_word and registered_word in " " + my_word:
-                    raise NameError(
-                        f"Cannot register '{my_word}'. "
-                        f"Currently, it is used in the rule: {rule}"
-                    )
-        self.rule_words[rxn_rule].append(" " + my_word)
+        if terminology is None:
+            terminology = {}
+        for rxn_rule in terminology.keys():
+            assert isinstance(rxn_rule, str)
+            if rxn_rule not in self.rule_words.keys():
+                raise ValueError(
+                    f"{rxn_rule} is not defined in reaction_rules.\n"
+                    f"Choose a reaction rule from {', '.join(map(str, self.rule_words.keys()))}"
+                )
+            for my_word in terminology[rxn_rule]:
+                assert isinstance(my_word, str)
+                for rule, words in self.rule_words.items():
+                    for registered_word in words:
+                        if " " + my_word in registered_word and registered_word in " " + my_word:
+                            raise NameError(
+                                f"Cannot supply '{my_word}' to '{rxn_rule}'. "
+                                f"Currently, it is used in the rule: {rule}"
+                            )
+                self.rule_words[rxn_rule].append(" " + my_word)
